/*
 * Copyright 2009 IHE International (http://www.ihe.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.ihe.gazelle.proxy.action;

import java.io.Serializable;
import java.util.Arrays;
import java.util.List;

import net.ihe.gazelle.proxy.netty.ChannelType;
import net.ihe.gazelle.proxy.netty.ConnectionConfig;
import net.ihe.gazelle.proxy.netty.ConnectionConfigSimple;

import org.apache.commons.lang.exception.ExceptionUtils;
import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Logger;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.faces.FacesMessages;
import org.jboss.seam.international.StatusMessage;
import org.jboss.seam.log.Log;
import org.jboss.seam.security.Identity;

@Name("channelManagerBean")
@Scope(ScopeType.PAGE)
public class ChannelManagerBean implements Serializable {

	private static final long serialVersionUID = 1879075873564126607L;

	/** Logger */
	@Logger
	private static Log log;

	@In
	private ProxyLocal proxyBean;

	@In
	Identity identity;

	private ChannelType messageType = ChannelType.HTTP;
	private Integer proxyPort = 10000;
	private String responderIP;
	private Integer responderPort;

	private int channelToStop;

	public ChannelManagerBean() {
		super();
	}

	public ChannelType getMessageType() {
		return messageType;
	}

	public List<ChannelType> getMessageTypes() {
		return Arrays.asList(ChannelType.values());
	}

	public Integer getProxyPort() {
		return proxyPort;
	}

	public String getResponderIP() {
		return responderIP;
	}

	public Integer getResponderPort() {
		return responderPort;
	}

	public void setChannelToStop(int proxyPort) {
		channelToStop = proxyPort;
	}

	public void setMessageType(ChannelType messageType) {
		this.messageType = messageType;
	}

	public void setProxyPort(Integer proxyPort) {
		this.proxyPort = proxyPort;
	}

	public void setResponderIP(String responderIP) {
		this.responderIP = responderIP;
	}

	public void setResponderPort(Integer responderPort) {
		this.responderPort = responderPort;
	}

	@SuppressWarnings("rawtypes")
	public String startChannel() {
		if (identity.hasRole("admin_role")) {
			ConnectionConfig connectionConfig = new ConnectionConfigSimple(getProxyPort(), getResponderIP(),
					getResponderPort(), getMessageType());
			try {
				proxyBean.startChannel(connectionConfig);
				FacesMessages.instance().add(StatusMessage.Severity.INFO, "Channel successfully created");
				return "/channels.xhtml";
			} catch (Throwable e) {
				log.error("Failed to start channel", e);
				FacesMessages.instance().add(StatusMessage.Severity.ERROR,
						"Failed to start channel... " + e.getMessage());
			}
		} else {
			FacesMessages.instance().add("Wrong role");
		}
		return null;
	}

	public void stopChannel(int localPort) {
		if (identity.hasRole("admin_role")) {
			try {
				proxyBean.stopChannel(localPort);
				FacesMessages.instance().add(StatusMessage.Severity.INFO, "Channel successfully stopped");
			} catch (Throwable e) {
				FacesMessages.instance().add(StatusMessage.Severity.ERROR,
						"Failed to stop channel : " + ExceptionUtils.getFullStackTrace(e));
			}
		} else {
			FacesMessages.instance().add("Wrong role");
		}
	}

	public void stopChannelReal() {
		stopChannel(channelToStop);
	}

}
